@@ -610,6 +610,7 @@ class nlmsg_base(dict):
     __slots__ = (
         "_buf",
         "data",
+        "chain",
         "offset",
         "length",
         "parent",
@@ -640,6 +641,7 @@ class nlmsg_base(dict):
         self.data = data or bytearray()
         self.offset = offset
         self.length = length or 0
+        self.chain = [self, ]
         if parent is not None:
             # some structures use parents, some not,
             # so don't create cycles without need
@@ -1140,6 +1142,38 @@ class nlmsg_base(dict):
         '''
         return [i[1] for i in self['attrs'] if i[0] == attr]
 
+    def nla(self, attr=None, default=NotInitialized):
+        '''
+        '''
+        if default is NotInitialized:
+            response = nlmsg_base()
+            del response['value']
+            del response['attrs']
+            response.value = None
+        chain = self.get('attrs', [])
+        if attr is not None:
+            chain = [i.nla for i in chain if i.name == attr]
+        else:
+            chain = [i.nla for i in chain]
+        if chain:
+            for link in chain:
+                link.chain = chain
+            response = chain[0]
+        return response
+
+    def __getattribute__(self, key):
+        try:
+            return super(nlmsg_base, self).__getattribute__(key)
+        except AttributeError:
+            if ord(key[0]) < 90:
+                return self.nla(key)
+            raise AttributeError(key)
+
+    def __getitem__(self, key):
+        if isinstance(key, int):
+            return self.chain[key]
+        return dict.__getitem__(self, key)
+
     def __setstate__(self, state):
         return self.load(state)
 
@@ -1467,6 +1501,19 @@ class nla_slot(object):
             return self.cell[1]._nla_flags
         return None
 
+    @property
+    def name(self):
+        return self.cell[0]
+
+    @property
+    def value(self):
+        return self.get_value()
+
+    @property
+    def nla(self):
+        self.try_to_decode()
+        return self.cell[1]
+
     def __getitem__(self, key):
         if key == 1:
             return self.get_value()
